"""The example package provides a simple example of how the 'attribox'
package can be used. """
#  AGPL-3.0 license
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._point import Point
from ._rectangle import Rectangle
